# Sample for GAPI CAN cyclic message definition and monitoring
# Needs min. Python 3.x.

import ctypes as ct

GAPI = ct.cdll.g_api
GAPI_common = ct.cdll.g_api_common
GAPI_can = ct.cdll.g_api_can

errorDesc = ct.create_string_buffer(4096)
errorDescLength = ct.c_uint(4096)

# open connection between g-api and firmware
print ("Open Interface")
CAN1 = b"CAN12"
porthandle = ct.c_uint(0xFFFFFFFF)
error = GAPI_common.G_Common_OpenInterface(CAN1, ct.byref(porthandle))
if error != 0:
        GAPI.G_GetErrorDescription2(0, error, ct.byref(errorDescLength), errorDesc)
        print ("Error G_Common_OpenInterface: 0x%X - %s" % (error, errorDesc.value.decode('utf-8')))

# initialize CAN interface
print ("Init Interface")
idMode = ct.c_ubyte(0)
InitInterface_CmdFlags = ct.c_uint(0)
error = GAPI_can.G_Can_InitInterface(porthandle, idMode, InitInterface_CmdFlags)
if  error != 0:
        GAPI.G_GetErrorDescription2(0, error, ct.byref(errorDescLength), errorDesc)
        print ("Error G_Can_InitInterface: 0x%X - %s" % (error, errorDesc.value.decode('utf-8')))

# configure a cyclic CAN message
print ("Cyclic Msg define")
Id = ct.c_uint(0x123)
cycleTime = ct.c_ushort(60)
send_state = ct.c_ubyte(0) # 0 = dont send
group = ct.c_ubyte(1)
msgCount = ct.c_ubyte(0)
Data = ct.c_ubyte * 8
MsgData = Data(1, 2, 3, 4, 5, 6, 7, 8)
dlc = ct.c_ubyte(8)
error = GAPI_can.G_Can_CyclicMsgs_DefineMsg(porthandle, Id, cycleTime, send_state, group, msgCount, MsgData, dlc)
if error != 0:
        GAPI.G_GetErrorDescription2(0, error, ct.byref(errorDescLength), errorDesc)
        print ("Error G_Can_InitInterface: 0x%X - %s" % (error, errorDesc.value.decode('utf-8')))

# start CAN monitor
print ("Start Buffermonitor")
monStart_cmdFlags = ct.c_uint(0x01 + 0x02 + 0x04 + 0x08 + 0x10) #see flags in g_api_can.h
error = GAPI_can.G_Can_Monitor_BufferMode_Start2(porthandle, monStart_cmdFlags, 1000)
if error != 0:
        GAPI.G_GetErrorDescription2(0, error, ct.byref(errorDescLength), errorDesc)
        print ("Error G_Can_Monitor_BufferMode_Start2: 0x%X - %s" % (error, errorDesc.value.decode('utf-8')))


input("enter to start")

# start sending of cyclic message
print ("Start Msg Group")
error = GAPI_can.G_Can_CyclicMsgs_StartGroup(porthandle, group)
if error != 0:
        GAPI.G_GetErrorDescription2(0, error, ct.byref(errorDescLength), errorDesc)
        print ("Error G_Can_InitInterface: 0x%X - %s" % (error, errorDesc.value.decode('utf-8')))


input("enter to stop")

# get monitored items
print ("Get Monitor Items")
class monItem(ct.Structure):
    _fields_ = [
            ('Timestamp', ct.c_int64),
            ('ID', ct.c_int),
            ('Flags', ct.c_byte),
            ('reserved', ct.c_byte),
            ('Dlc', ct.c_byte),
            ('DataLength', ct.c_byte),
            ('Data', ct.c_byte*64)
            ]
monGet_nrItems = ct.c_uint(100) # buffer size = 100 items
monGet_items = (monItem * 100)()
monGet_rspFlags = ct.c_uint(0)
error = GAPI_can.G_Can_Monitor_BufferMode_GetItems_Large(porthandle, ct.byref(monGet_rspFlags), ct.byref(monGet_nrItems), monGet_items )
if error != 0:
        GAPI.G_GetErrorDescription2(0, error, ct.byref(errorDescLength), errorDesc)
        print ("Error G_Can_Monitor_BufferMode_GetItems_Large: 0x%X - %s" % (error, errorDesc.value.decode('utf-8')))

# print monitor items in loop
print ("Monitor response flags: 0x%X" % monGet_rspFlags.value) 
for i in range(monGet_nrItems.value):
        print ("TS: %d µs ID: 0x%X DLC: %d Flags: 0x%X Data: " %( (monGet_items[i].Timestamp / 1000), monGet_items[i].ID, monGet_items[i].Dlc, monGet_items[i].Flags), end="")
        for j in range(monGet_items[i].DataLength):
                print ("%02X " %monGet_items[i].Data[j], end="")
        print ("")
        
# stop cyclic message
print ("Stop Msg Group")
error = GAPI_can.G_Can_CyclicMsgs_StopGroup(porthandle, group)
if error != 0:
        GAPI.G_GetErrorDescription2(0, error, ct.byref(errorDescLength), errorDesc)
        print ("Error G_Can_InitInterface: 0x%X - %s" % (error, errorDesc.value.decode('utf-8')))

# stop bus monitor
print ("Stop Buffermonitor")
error = GAPI_can.G_Can_Monitor_BufferMode_Stop(porthandle)
if error != 0:
        GAPI.G_GetErrorDescription2(0, error, ct.byref(errorDescLength), errorDesc)
        print ("Error G_Can_Monitor_BufferMode_Stop: 0x%X - %s" % (error, errorDesc.value.decode('utf-8')))

# close connection between g-api and firmware    
print ("Close porthandle")
error = GAPI_common.G_Common_CloseInterface(porthandle)
if error != 0:
        GAPI.G_GetErrorDescription2(0, error, ct.byref(errorDescLength), errorDesc)
        print ("Error G_Can_InitInterface: 0x%X - %s" % (error, errorDesc.value.decode('utf-8')))
